import React, {Component} from 'react'
import PropTypes from 'prop-types'
import {
    ActivityIndicator,
    RefreshControl,
    ScrollView,
    Platform,
    Text,
    FlatList,
    View
} from 'react-native'
import styles from './style'
import RefreshableHeard from './refreshableheard/index'
const PaginationStatus = {
    custormHeardView: 3,
    firstLoad: 0,
    waiting: 1,
    allLoaded: 2
};
export default class CusLists extends Component {
    static defaultProps = {
        initialNumToRender: 10,
        horizontal: false,
        firstLoader: true,
        scrollEnabled: true,
        method: null,
        params: {},
        enableEmptySections: true,
        // Custom View
        ListHeaderComponent: null,
        renderItem: null,
        paginatiAPINetView: null,
        paginationAllLoadedView: null,
        paginationWaitingView: null,
        ListEmptyComponent: null,
        separator: null,
        // Refreshable
        refreshable: true,
        refreshableMode: Platform.OS === 'ios' ? 'ios_base' : 'android_base', // android_base or ios_base
        // RefreshControl
        refreshableTitle: null,
        refreshableColors: ['dimgray', 'tomato', 'limegreen'],
        refreshableProgressBackgroundColor: 'white',
        refreshableSize: undefined,
        refreshableTintColor: 'lightgray',
        customRefreshControl: null,
        // ios_base RefreshView
        customRefreshView: null,
        displayDate: true,
        arrowImageStyle: undefined,
        // Pagination
        pagination: true,
        autoPagination: true,
        allLoadedText: '没有加载更多了',
        // Spinner
        spinnerColor: undefined,
        fetchingSpinnerSize: 'large',
        waitingSpinnerSize: 'small',
        waitingSpinnerText: '加载中',
        // GridView
        numColumns: 1
    };

    static propTypes = {
        params: PropTypes.object,//服务需要的数据传递
        initialNumToRender: PropTypes.number,
        horizontal: PropTypes.bool,
        firstLoader: PropTypes.bool,
        scrollEnabled: PropTypes.bool,
        method: PropTypes.func,
        enableEmptySections: PropTypes.bool,
        // Custom ListView
        ListHeaderComponent: PropTypes.any,
        renderItem: PropTypes.any,
        paginatiAPINetView: PropTypes.any,
        paginationAllLoadedView: PropTypes.any,
        paginationWaitingView: PropTypes.func,
        ListEmptyComponent: PropTypes.any,
        separator: PropTypes.func,
        // Refreshable
        refreshable: PropTypes.bool,
        refreshableMode: PropTypes.string,
        // RefreshControl
        refreshableTitle: PropTypes.string,
        refreshableColors: PropTypes.array,
        refreshableProgressBackgroundColor: PropTypes.string,
        refreshableSize: PropTypes.string,
        refreshableTintColor: PropTypes.string,
        customRefreshControl: PropTypes.func,
        // ios_base RefreshView
        customRefreshView: PropTypes.func,
        displayDate: PropTypes.bool,
        arrowImageStyle: PropTypes.object,
        // Pagination
        pagination: PropTypes.bool,
        autoPagination: PropTypes.bool,
        allLoadedText: PropTypes.string,
        // Spinner
        spinnerColor: PropTypes.string,
        fetchingSpinnerSize: PropTypes.any,
        waitingSpinnerSize: PropTypes.any,
        waitingSpinnerText: PropTypes.string,
        // GridView
        numColumns: PropTypes.number
    };

    constructor(props) {
        super(props);
        this.setPage(1);
        this.setRows([]);

        this.state = {
            dataSource: [],
            isRefreshing: false,
            paginationStatus: PaginationStatus.firstLoad
        };
        this.updateRows = this.updateRows.bind(this);
        this.renderRefreshControl = this.renderRefreshControl.bind(this);
        this.errorApi = this.errorApi.bind(this);
        this.onPaginate = this.onPaginate.bind(this);
        this.onEndReached = this.onEndReached.bind(this);
        this.onRefresh = this.onRefresh.bind(this);
        this.paginatiAPINetView = this.paginatiAPINetView.bind(this);
        this.paginationAllLoadedView = this.paginationAllLoadedView.bind(this);
        this.paginationWaitingView = this.paginationWaitingView.bind(this);
        this.renderHeader = this.renderHeader.bind(this);
        this.renderItem = this.renderItem.bind(this);
        this.renderSeparator = this.renderSeparator.bind(this);
        this.renderEmptyView = this.renderEmptyView.bind(this);
        this.renderFooter = this.renderFooter.bind(this);
    }

    //第一次进来的数据加载
    componentDidMount() {
        this.mounted = true;
        if (this.props.firstLoader) {
            this.props.method(this.getPage(), this.postRefresh, this.errorApi)
        }
    }

    componentWillUnmount() {
        this.mounted = false
    }

    //重新刷新动作
    onRefresh() {
        if (this.mounted) {
            this.setState({
                isRefreshing: true
            });
            this.setPage(1);
            this.props.method(this.getPage(), this.postRefresh, this.errorApi)
        }
    };

    //数据的加载更多操作
    onPaginate() {
        if (this.state.paginationStatus !== PaginationStatus.allLoaded && !this.state.isRefreshing) {
            this.setState({paginationStatus: PaginationStatus.waiting});
            this.props.method(this.getPage() + 1, this.postPaginate, this.errorApi)
        }
    };

    //动到距离底部（XX）距离时触发的
    onEndReached() {
        if (this.props.pagination && this.props.autoPagination && this.state.paginationStatus === PaginationStatus.waiting) {
            this.onPaginate();
        }
    };

    setPage = page => this.page = page;

    getPage = () => this.page;

    setIsLast = isLast => this.isLast = isLast;

    getIsLast = () => this.isLast;

    setRows = rows => this.rows = rows;

    getRows = () => this.rows;

    scrollToOffset = (option) => {
        this._flatList.scrollToOffset(option)
    };

    scrollToIndex = (option) => {
        this._flatList.scrollToIndex(option)
    };

    scrollToItem = (option) => {
        this._flatList.scrollToItem(option)
    };

    scrollToEnd = (option) => {
        this._flatList.scrollToEnd(option)
    };
    //第一次加载数据判断
    postRefresh = (rows = [], pageLimit, pageSize) => {
        if (this.mounted) {
            let paginationStatus = PaginationStatus.waiting;
            if (rows.length < pageSize) {
                paginationStatus = PaginationStatus.allLoaded
            }
            this.updateRows(rows, paginationStatus)
        }
    };

    //错误的提示以及操作
    errorApi() {
        if (this.mounted) {
            console.log("Close");
            this.setState({isRefreshing: false});
            if (this.props.refreshableMode === 'ios_base' && this._flatList._listRef._scrollRef.onRefreshEnd) {
                this._flatList._listRef._scrollRef.onRefreshEnd()
            }
        }
    };

    //加载数据之后进行合并和判断
    postPaginate = (rows = [], pageLimit, pageSize) => {
        this.setPage(this.getPage() + 1);
        console.log(this.getIsLast());
        let mergedRows;
        let paginationStatus;
        if (rows.length === 0) {
            paginationStatus = PaginationStatus.allLoaded
        } else if (rows.length < pageSize && !this.getIsLast()) {
            this.setIsLast(true);
            mergedRows = this.getRows().concat(rows);
            paginationStatus = PaginationStatus.waiting
        } else if (this.getIsLast()) {
            this.setIsLast(false);
            paginationStatus = PaginationStatus.allLoaded
        } else {
            mergedRows = this.getRows().concat(rows);
            paginationStatus = PaginationStatus.waiting
        }

        this.updateRows(mergedRows, paginationStatus)
    };

    //判断完之后进行一个数据设置和刷新关闭
    updateRows(rows, paginationStatus) {
        if (rows) {
            this.setRows(rows);
            this.setState({
                dataSource: rows,
                isRefreshing: false,
                paginationStatus
            })
        } else {
            this.setState({
                dataSource: this.getRows().slice(),
                isRefreshing: false,
                paginationStatus
            })
        }

        if (this.props.refreshableMode === 'ios_base') {
            this.errorApi()
        }
    };

    updateDataSource(rows = []) {
        this.setRows(rows);
        this.setState({
            dataSource: rows
        })
    }

    //ios的样式设置，android模仿ios
    renderScrollComponent = (props) => {
        if (this.props.refreshable && this.props.refreshableMode === 'ios_base') {
            return (
                <RefreshableHeard
                    {...props}

                    onRefresh={this.onRefresh}
                    ref={ref => this.scrollView = ref}
                />
            )
        }
        return (
            <ScrollView
                {...props}
                ref={ref => this.scrollView = ref}
            />
        )
    };

    //第一次网络请求的刷新动画（虚假的。没有）
    paginatiAPINetView() {
        if (this.props.paginatiAPINetView) {
            return this.fucChange(this.props.paginatiAPINetView)
        }
        return null;
    };

    //设置默认的加载完毕
    paginationAllLoadedView() {
        if (this.props.pagination) {
            if (this.props.paginationAllLoadedView) {
                return this.fucChange(this.props.paginationAllLoadedView)
            }
            return (
                <View style={styles.paginationView}>
                    <Text style={styles.allLoadedText}>
                        {this.props.allLoadedText}
                    </Text>
                </View>
            )
        }

        return null
    };

    //设置默认的加载更多
    paginationWaitingView(paginateCallback) {
        if (this.props.horizontal) {
            return null;
        }
        if (this.props.pagination) {
            if (this.props.autoPagination) {
                if (this.props.paginationWaitingView) {
                    return this.props.paginationWaitingView(paginateCallback)
                }
                return (
                    <View style={styles.paginationView}>
                        <ActivityIndicator color={this.props.spinnerColor} size={this.props.waitingSpinnerSize}/>
                        <Text
                            style={[styles.paginationViewText, {marginLeft: 5}]}
                        >{this.props.waitingSpinnerText}
                        </Text>
                    </View>
                )
            }
        }

        return null
    };

    //添加头部组件
    renderHeader() {
        if (this.props.ListHeaderComponent) {
            return this.fucChange(this.props.ListHeaderComponent)
        }
        return null
    };

    renderItem({item, index, separators}) {
        if (this.props.renderItem) {
            return this.fucChange(this.props.renderItem, item, index, separators)
        }
        return null
    };

    fucChange = (WrappedComponent, item, index, separators) => {
        return (
            <WrappedComponent item={item} index={index} separators={separators}
                              {...this.props}/>
        )
    }

    renderSeparator() {
        if (this.props.separator) {
            if (this.props.numColumns > 1) {
                return null
            }

            return this.props.separator()
        }

        return null
    };

    //判断空页面
    renderEmptyView() {
        if (this.props.horizontal) {
            return null;
        }
        if (this.state.paginationStatus !== PaginationStatus.firstLoad && this.props.ListEmptyComponent) {
            return this.fucChange(this.props.ListEmptyComponent)
        }
        return null
    };


    renderFooter() {
        if (this.state.paginationStatus === PaginationStatus.firstLoad) {
            return this.paginatiAPINetView()
        } else if (this.state.paginationStatus === PaginationStatus.waiting && this.props.autoPagination === false) {
            return this.paginationWaitingView(this.onPaginate)
        } else if (this.state.paginationStatus === PaginationStatus.waiting && this.props.autoPagination === true) {
            return this.paginationWaitingView()
        }
        else if (this.getRows().length !== 0 && this.state.paginationStatus === PaginationStatus.allLoaded) {
            return this.paginationAllLoadedView()
        }
        return null
    };

    //刷新控制
    renderRefreshControl() {
        if (this.props.refreshable && this.props.refreshableMode === 'android_base') {
            if (this.props.customRefreshControl) {
                return this.props.customRefreshControl(this.state.isRefreshing, this.onRefresh)
            }

            return (
                <RefreshControl
                    onRefresh={this.onRefresh}
                    refreshing={this.state.isRefreshing}
                    colors={this.props.refreshableColors}
                    progressBackgroundColor={this.props.refreshableProgressBackgroundColor}
                    size={this.props.refreshableSize}
                    tintColor={this.props.refreshableTintColor}
                    title={this.props.refreshableTitle}
                />
            )
        }

        return null
    };

    render() {
        const {numColumns} = this.props;
        let state = this.state;
        return (
            <FlatList
                renderScrollComponent={this.renderScrollComponent}
                onRefresh={this.onRefresh}
                refreshing={state.isRefreshing}
                key={numColumns}
                keyExtractor={(item, index) => `${index} - ${item}`} // 这个是对应的请求，必填
                onEndReachedThreshold={0.1}
                {...this.props}
                ref={ref => this._flatList = ref}
                data={state.dataSource}
                renderItem={this.renderItem}
                ItemSeparatorComponent={this.renderSeparator}
                ListHeaderComponent={this.renderHeader}
                ListFooterComponent={this.renderFooter}
                ListEmptyComponent={this.renderEmptyView}
                onEndReached={this.onEndReached}
                refreshControl={this.renderRefreshControl()}
                numColumns={numColumns}
            />
        )
    }

}


