import React from 'react'
import {
    ActivityIndicator,
    Animated,
    AsyncStorage,
    Dimensions,
    Easing,
    ScrollView,
    Text,
    View
} from 'react-native'
import dateFormat from './util'
const {height } = Dimensions.get('window');
import styles from './style'
const DATE_KEY = 'RefreshDate';
const RefreshStatus = {
    pullToRefresh: 0,
    releaseToRefresh: 1,
    refreshing: 2
};
const PaginationStatus = {
    firstLoad: 0,
    waiting: 1,
    allLoaded: 2
};

export default class RefreshableHeard extends ScrollView {
    static defaultProps = {
        horizontal: false,
        scrollEnabled: true,
        header: null,
        refreshable: true,
        refreshableTitlePull: '下拉刷新',
        refreshableTitleRefreshing: '加载中...',
        refreshableTitleRelease: '下拉刷新中...',
        customRefreshView: null,
        displayDate: false,
        dateFormat: 'yyyy-MM-dd hh:mm',
        dateTitle: '刷新时间: ',
        arrowImageSource: require('./downArrow.png'),
        refreshViewStyle: undefined,
        dateStyle: undefined,
        refreshViewHeight: 80,
        insideOfUltimateListView: false
    };

    _offsetY = 0;
    _isRefreshing = false;
    _dragFlag = false;

    constructor(props) {
        super(props);
        this.state = {
            arrowAngle: new Animated.Value(0),
            refreshStatus: RefreshStatus.refreshing,
            refreshTitle: this.props.refreshableTitleRefreshing,
            date: this.props.date,
            showRefreshHeader: false
        };
        this.onScroll = this.onScroll.bind(this);
    }

    async componentDidMount() {
        try {
            let result = await AsyncStorage.getItem(DATE_KEY);
            if (result) {
                result = parseInt(result, 10);
                this.setState({
                    date: dateFormat(new Date(result), this.props.dateFormat)
                })
            } else {
                this.setState({
                    date: dateFormat(new Date(), this.props.dateFormat)
                })
            }
        } catch (err) {
            console.log(err)
        }
    }

    onScroll(event){

        const {y} = event.nativeEvent.contentOffset;
        const {refreshViewHeight} = this.props;
        if (y <= refreshViewHeight) {
            this._offsetY = y - refreshViewHeight
        }
        console.log("onScroll --------------------" +this._dragFlag + (y));
        if (this._dragFlag) {
            if (!this._isRefreshing) {
                if (y <= 10) {
                    if (this.state.refreshStatus !== RefreshStatus.releaseToRefresh) {
                        this.setState({
                            refreshStatus: RefreshStatus.releaseToRefresh,
                            refreshTitle: this.props.refreshableTitleRelease
                        });
                        Animated.timing(this.state.arrowAngle, {
                            toValue: 1,
                            duration: 50,
                            easing: Easing.inOut(Easing.quad)
                        }).start()
                    }
                } else if (this.state.refreshStatus !== RefreshStatus.pullToRefresh) {
                    this.setState({
                        refreshStatus: RefreshStatus.pullToRefresh,
                        refreshTitle: this.props.refreshableTitlePull
                    });
                    Animated.timing(this.state.arrowAngle, {
                        toValue: 0,
                        duration: 50,
                        easing: Easing.inOut(Easing.quad)
                    }).start()
                }
            }
        } else if (y <= 5) {
            console.log('refreshViewHeight()4'+refreshViewHeight);
            setTimeout(() => this._scrollview.scrollTo({x: 0, y: refreshViewHeight, animated: false}), 100)
        }
        if (this.props.onScroll) {
            this.props.onScroll(event)
        }
    };

    onScrollBeginDrag = (event) => {

        this._dragFlag = true;
        const {refreshViewHeight} = this.props;
        this.setState({showRefreshHeader:true});
        this._offsetY = event.nativeEvent.contentOffset.y - refreshViewHeight;
        console.log('onScrollBeginDrag()');
        if (this.props.onScrollBeginDrag) {
            this.props.onScrollBeginDrag(event)
        }
    };

    onScrollEndDrag = (event) => {
        this._dragFlag = false;
        const {y} = event.nativeEvent.contentOffset;
        console.log('onScrollEndDrag()'+y);
        const {refreshViewHeight} = this.props;
        this._offsetY = y - refreshViewHeight;
        // console.log('onScrollEndDrag()' + y);
        if (!this._isRefreshing) {
            if (this.state.refreshStatus === RefreshStatus.releaseToRefresh) {
                this._isRefreshing = true;
                this.setState({
                    refreshStatus: RefreshStatus.refreshing,
                    refreshTitle: this.props.refreshableTitleRefreshing
                });
                console.log('refreshViewHeight()3'+refreshViewHeight);
                this._scrollview.scrollTo({x: 0, y: 0, animated: true});
                if (this.props.insideOfUltimateListView) {
                    this.props.onRefresh()
                } else {
                    this.props.onRefresh(() => {
                        this.onRefreshEnd()
                    })
                }
            } else if (y <= refreshViewHeight) {
                console.log('refreshViewHeight()'+refreshViewHeight);
                this._scrollview.scrollTo({x: 0, y: refreshViewHeight, animated: true})
            }
        } else if (y <= refreshViewHeight) {
            console.log('refreshViewHeight()2'+refreshViewHeight);
            this._scrollview.scrollTo({x: 0, y: 0, animated: true})
        }
        if (this.props.onScrollEndDrag) {
            this.props.onScrollEndDrag(event)
        }
    };

    scrollTo = (option) => {
        this._scrollview.scrollTo(option)
    };

    scrollToEnd = (option) => {
        this._scrollview.scrollToEnd(option)
    };

    onRefreshEnd = () => {
        console.log('onRefreshEnd()');
        if (this.state.refreshStatus === RefreshStatus.refreshing) {
            this._isRefreshing = false;
            const now = new Date().getTime();
            this.setState({
                showRefreshHeader: false,
                refreshStatus:RefreshStatus.pullToRefresh
            });
            setTimeout(() => {
                if (this._scrollview.scrollTo) {
                    this._scrollview.scrollTo({x: 0, y: this.props.refreshViewHeight, animated: true})
                }
                this.setState({
                    refreshStatus: RefreshStatus.pullToRefresh,
                    refreshTitle: this.props.refreshableTitlePull,
                    date: dateFormat(now, this.props.dateFormat)
                })
            }, 1000);

            AsyncStorage.setItem(DATE_KEY, now.toString());
            Animated.timing(this.state.arrowAngle, {
                toValue: 0,
                duration: 50,
                easing: Easing.inOut(Easing.quad)
            }).start()
        }
    };

    renderRefreshHeader() {
        console.log('onScroll()------'+this.state.refreshStatus + this.state.showRefreshHeader);
        if(!this.state.showRefreshHeader){
            return null;
        }
        if (this.props.customRefreshView) {
            return (
                <View style={[styles.header, this.props.refreshViewStyle]}>
                    {this.props.customRefreshView(this.state.refreshStatus, this._offsetY)}
                </View>
            )
        }

        return (
            <View
                style={[styles.header, this.props.refreshViewStyle, {height: this.props.refreshViewHeight}]}
            >
                <View style={styles.status}>
                    {this.renderSpinner()}
                    <Text style={styles.statusTitle}>{this.state.refreshTitle}</Text>
                </View>
                {this.props.displayDate &&
                <Text
                    style={[styles.date, this.props.dateStyle]}
                >{this.props.dateTitle + this.state.date}
                </Text>
                }
            </View>
        )
    }

    renderSpinner() {
        console.log('onScroll()------2222'+this.state.refreshStatus + this.state.showRefreshHeader);
        if(!this.state.showRefreshHeader){
            return null;
        }
        if (this.state.refreshStatus === RefreshStatus.refreshing) {
            return (
                <ActivityIndicator style={{marginRight: 10}}/>
            )
        }
        return (
            <Animated.Image
                source={this.props.arrowImageSource}
                resizeMode="contain"
                style={[styles.arrow,
                    this.props.arrowImageStyle,
                    {
                        transform: [{
                            rotateX: this.state.arrowAngle.interpolate({
                                inputRange: [0, 1],
                                outputRange: ['0deg', '-180deg']
                            })
                        }]
                    }]}
            />
        )
    }

    render() {
        return (
            <ScrollView
                ref={c => this._scrollview = c}
                {...this.props}
                scrollEventThrottle={16}
                onScroll={this.onScroll}
                contentContainerStyle={{ minHeight: height }}
                // onMomentumScrollEnd={this.onScrollEndDrag}
                onScrollEndDrag={this.onScrollEndDrag}
                onScrollBeginDrag={this.onScrollBeginDrag}
            >
                {this.renderRefreshHeader()}
                {this.props.children}
            </ScrollView>
        )
    }
}


